#include "as_config.h"
#include "file_util.hh"
#include "file_exceps.hh"

#include <cstdio>

#ifdef USE_FILE_LOCKS

// POSIX headers
#include <fcntl.h>

#endif

#include <unistd.h>
#include <sys/stat.h>

#include <fstream>

namespace autil {

  void open_file(ifstream & in, const string & file, int mode = ios::in) {
    in.open(file.c_str(), mode);
    if (in.rdstate() & ios::badbit )
      throw CantReadFile(file);
  }
    
  string add_possible_dir(const string & dir, const string & file) {
      if (file.size() && file[0] == '/' || (file[0] == '.' 
					    && file.size() > 1
					    && file[1] == '/'))
	return file;
      else
	return dir + "/" + file;
  }
   
  int filedesc(fstream & f) {
     return f.rdbuf()->fd();
  }

  time_t get_modification_time(fstream & f) {
    struct stat s;
    fstat(filedesc(f), &s);
    return s.st_mtime;
  }

  void open_file_readlock(fstream & in, const string & file) {
    in.open(file.c_str(), ios::in);
    if (in.rdstate() & ios::badbit )
      throw CantReadFile(file);
#ifdef USE_FILE_LOCKS 
    int fd = filedesc(in);
    struct flock fl;
    fl.l_type   = F_RDLCK;
    fl.l_whence = SEEK_SET;
    fl.l_start  = 0;
    fl.l_len    = 0;
    if (fcntl(fd, F_SETLKW, &fl) == -1) {
      in.close();
      throw CantReadFile(file);
    }
#endif
  }

  bool open_file_writelock(fstream & inout, const string & file) {
#ifndef USE_FILE_LOCKS
    bool exists = file_exists(file);
#endif
    inout.open(file.c_str(), ios::in | ios::out);
    if (inout.rdstate() & ios::badbit )
      throw CantWriteFile(file);
#ifdef USE_FILE_LOCKS 
    int fd = filedesc(inout);
    struct flock fl;
    fl.l_type   = F_WRLCK;
    fl.l_whence = SEEK_SET;
    fl.l_start  = 0;
    fl.l_len    = 0;
    if (fcntl(fd, F_SETLKW, &fl) == -1)
      throw CantWriteFile(file);
    struct stat s;
    fstat(fd, &s);
    return s.st_size != 0;
#else
    return exists;
#endif
  }

  void truncate_file(fstream & f, const string & name) {
#ifdef USE_FILE_LOCKS
    f.flush();
    f.clear();
    f.seekp(0, ios::beg);
    ftruncate(filedesc(f),0);
#else
    f.close();
    f.open(name.c_str(), ios::in | ios::out | ios::trunc);
#endif    
  }
  
  bool remove_file(const string & name) {
    return remove(name.c_str()) == 0;
  }

  bool file_exists(const string & name) {
    return access(name.c_str(), F_OK) == 0;
  }

  bool rename_file(const string & orig_name, const string & new_name) 
  {
    remove(new_name.c_str());
    return rename(orig_name.c_str(), new_name.c_str()) == 0;
  }

}

